/***********************************************
/gen auto by zuitools
***********************************************/
#include "DeviceUpdateActivity.h"

/*TAG:GlobalVariable全局变量*/
static ZKButton* mButton1Ptr;
static ZKTextView* mVersionNoPtr;
static ZKTextView* mTextView7Ptr;
static ZKTextView* mTextViewTitlePtr;
static ZKTextView* mVersionPtr;
static ZKTextView* mTextView1Ptr;
static ZKButton* mButtonUpdatePtr;
static ZKTextView* mTextView2Ptr;
static ZKWindow* mWindow1Ptr;
static ZKButton* mButtonDialog1Ptr;
static ZKTextView* mTextViewDialog1Ptr;
static ZKTextView* mTextView6Ptr;
static ZKWindow* mWindowDialog1Ptr;
static ZKTextView* mTextViewProgressMessagePtr;
static ZKTextView* mTextViewProgressLoadingPtr;
static ZKTextView* mTextView4Ptr;
static ZKWindow* mWindowProgressPtr;
static ZKTextView* mVersionNoNewPtr;
static ZKTextView* mTextView8Ptr;
static ZKButton* mButtonInstantlyPtr;
static ZKTextView* mTextView3Ptr;
static ZKTextView* mVersionNewPtr;
static ZKTextView* mTextView5Ptr;
static ZKWindow* mWindowFindPtr;
static DeviceUpdateActivity* mActivityPtr;

/*register activity*/
REGISTER_ACTIVITY(DeviceUpdateActivity);

typedef struct {
	int id; // 定时器ID ， 不能重复
	int time; // 定时器  时间间隔  单位 毫秒
}S_ACTIVITY_TIMEER;

#include "logic/DeviceUpdateLogic.cc"

/***********/
typedef struct {
    int id;
    const char *pApp;
} SAppInfo;

/**
 *点击跳转window
 */
static SAppInfo sAppInfoTab[] = {
//  { ID_MAIN_TEXT, "TextViewActivity" },
};

/***************/
typedef bool (*ButtonCallback)(ZKButton *pButton);
/**
 * button onclick表
 */
typedef struct {
    int id;
    ButtonCallback callback;
}S_ButtonCallback;

/*TAG:ButtonCallbackTab按键映射表*/
static S_ButtonCallback sButtonCallbackTab[] = {
    ID_DEVICEUPDATE_Button1, onButtonClick_Button1,
    ID_DEVICEUPDATE_ButtonUpdate, onButtonClick_ButtonUpdate,
    ID_DEVICEUPDATE_ButtonDialog1, onButtonClick_ButtonDialog1,
    ID_DEVICEUPDATE_ButtonInstantly, onButtonClick_ButtonInstantly,
};
/***************/


typedef void (*SeekBarCallback)(ZKSeekBar *pSeekBar, int progress);
typedef struct {
    int id;
    SeekBarCallback callback;
}S_ZKSeekBarCallback;
/*TAG:SeekBarCallbackTab*/
static S_ZKSeekBarCallback SZKSeekBarCallbackTab[] = {
};


typedef int (*ListViewGetItemCountCallback)(const ZKListView *pListView);
typedef void (*ListViewobtainListItemDataCallback)(ZKListView *pListView,ZKListView::ZKListItem *pListItem, int index);
typedef void (*ListViewonItemClickCallback)(ZKListView *pListView, int index, int id);
typedef struct {
    int id;
    ListViewGetItemCountCallback getListItemCountCallback;
    ListViewobtainListItemDataCallback obtainListItemDataCallback;
    ListViewonItemClickCallback onItemClickCallback;
}S_ListViewFunctionsCallback;
/*TAG:ListViewFunctionsCallback*/
static S_ListViewFunctionsCallback SListViewFunctionsCallbackTab[] = {
};


typedef void (*SlideWindowItemClickCallback)(ZKSlideWindow *pSlideWindow, int index);
typedef struct {
    int id;
    SlideWindowItemClickCallback onSlideItemClickCallback;
}S_SlideWindowItemClickCallback;
/*TAG:SlideWindowFunctionsCallbackTab*/
static S_SlideWindowItemClickCallback SSlideWindowItemClickCallbackTab[] = {
};


typedef void (*EditTextInputCallback)(const std::string &text);
typedef struct {
    int id;
    EditTextInputCallback onEditTextChangedCallback;
}S_EditTextInputCallback;
/*TAG:EditTextInputCallback*/
static S_EditTextInputCallback SEditTextInputCallbackTab[] = {
};

typedef void (*VideoViewCallback)(ZKVideoView *pVideoView, int msg);
typedef struct {
    int id; //VideoView ID
    bool loop; // 是否是轮播类型
    int defaultvolume;//轮播类型时,默认视频音量
    VideoViewCallback onVideoViewCallback;
}S_VideoViewCallback;
/*TAG:VideoViewCallback*/
static S_VideoViewCallback SVideoViewCallbackTab[] = {
};


DeviceUpdateActivity::DeviceUpdateActivity() {
	//todo add init code here
	mVideoLoopIndex = -1;
	mVideoLoopErrorCount = 0;
}

DeviceUpdateActivity::~DeviceUpdateActivity() {
  //todo add init file here
  // 退出应用时需要反注册
    EASYUICONTEXT->unregisterGlobalTouchListener(this);
    onUI_quit();
    unregisterProtocolDataUpdateListener(onProtocolDataUpdate);
    mButton1Ptr = NULL;
    mVersionNoPtr = NULL;
    mTextView7Ptr = NULL;
    mTextViewTitlePtr = NULL;
    mVersionPtr = NULL;
    mTextView1Ptr = NULL;
    mButtonUpdatePtr = NULL;
    mTextView2Ptr = NULL;
    mWindow1Ptr = NULL;
    mButtonDialog1Ptr = NULL;
    mTextViewDialog1Ptr = NULL;
    mTextView6Ptr = NULL;
    mWindowDialog1Ptr = NULL;
    mTextViewProgressMessagePtr = NULL;
    mTextViewProgressLoadingPtr = NULL;
    mTextView4Ptr = NULL;
    mWindowProgressPtr = NULL;
    mVersionNoNewPtr = NULL;
    mTextView8Ptr = NULL;
    mButtonInstantlyPtr = NULL;
    mTextView3Ptr = NULL;
    mVersionNewPtr = NULL;
    mTextView5Ptr = NULL;
    mWindowFindPtr = NULL;
    mActivityPtr = NULL;
}

const char* DeviceUpdateActivity::getAppName() const{
	return "DeviceUpdate.ftu";
}

//TAG:onCreate
void DeviceUpdateActivity::onCreate() {
	Activity::onCreate();
    mButton1Ptr = (ZKButton*)findControlByID(ID_DEVICEUPDATE_Button1);
    mVersionNoPtr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_VersionNo);
    mTextView7Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView7);
    mTextViewTitlePtr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextViewTitle);
    mVersionPtr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_Version);
    mTextView1Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView1);
    mButtonUpdatePtr = (ZKButton*)findControlByID(ID_DEVICEUPDATE_ButtonUpdate);
    mTextView2Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView2);
    mWindow1Ptr = (ZKWindow*)findControlByID(ID_DEVICEUPDATE_Window1);
    mButtonDialog1Ptr = (ZKButton*)findControlByID(ID_DEVICEUPDATE_ButtonDialog1);
    mTextViewDialog1Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextViewDialog1);
    mTextView6Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView6);
    mWindowDialog1Ptr = (ZKWindow*)findControlByID(ID_DEVICEUPDATE_WindowDialog1);
    mTextViewProgressMessagePtr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextViewProgressMessage);
    mTextViewProgressLoadingPtr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextViewProgressLoading);
    mTextView4Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView4);
    mWindowProgressPtr = (ZKWindow*)findControlByID(ID_DEVICEUPDATE_WindowProgress);
    mVersionNoNewPtr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_VersionNoNew);
    mTextView8Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView8);
    mButtonInstantlyPtr = (ZKButton*)findControlByID(ID_DEVICEUPDATE_ButtonInstantly);
    mTextView3Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView3);
    mVersionNewPtr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_VersionNew);
    mTextView5Ptr = (ZKTextView*)findControlByID(ID_DEVICEUPDATE_TextView5);
    mWindowFindPtr = (ZKWindow*)findControlByID(ID_DEVICEUPDATE_WindowFind);
	mActivityPtr = this;
	onUI_init();
    registerProtocolDataUpdateListener(onProtocolDataUpdate); 
    rigesterActivityTimer();
}

void DeviceUpdateActivity::onClick(ZKBase *pBase) {
	//TODO: add widget onClik code 
    int buttonTablen = sizeof(sButtonCallbackTab) / sizeof(S_ButtonCallback);
    for (int i = 0; i < buttonTablen; ++i) {
        if (sButtonCallbackTab[i].id == pBase->getID()) {
            if (sButtonCallbackTab[i].callback((ZKButton*)pBase)) {
            	return;
            }
            break;
        }
    }


    int len = sizeof(sAppInfoTab) / sizeof(sAppInfoTab[0]);
    for (int i = 0; i < len; ++i) {
        if (sAppInfoTab[i].id == pBase->getID()) {
            EASYUICONTEXT->openActivity(sAppInfoTab[i].pApp);
            return;
        }
    }

	Activity::onClick(pBase);
}

void DeviceUpdateActivity::onResume() {
	Activity::onResume();
	EASYUICONTEXT->registerGlobalTouchListener(this);
	startVideoLoopPlayback();
	onUI_show();
}

void DeviceUpdateActivity::onPause() {
	Activity::onPause();
	EASYUICONTEXT->unregisterGlobalTouchListener(this);
	stopVideoLoopPlayback();
	onUI_hide();
}

void DeviceUpdateActivity::onIntent(const Intent *intentPtr) {
	Activity::onIntent(intentPtr);
	onUI_intent(intentPtr);
}

bool DeviceUpdateActivity::onTimer(int id) {
	return onUI_Timer(id);
}

void DeviceUpdateActivity::onProgressChanged(ZKSeekBar *pSeekBar, int progress){

    int seekBarTablen = sizeof(SZKSeekBarCallbackTab) / sizeof(S_ZKSeekBarCallback);
    for (int i = 0; i < seekBarTablen; ++i) {
        if (SZKSeekBarCallbackTab[i].id == pSeekBar->getID()) {
            SZKSeekBarCallbackTab[i].callback(pSeekBar, progress);
            break;
        }
    }
}

int DeviceUpdateActivity::getListItemCount(const ZKListView *pListView) const{
    int tablen = sizeof(SListViewFunctionsCallbackTab) / sizeof(S_ListViewFunctionsCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SListViewFunctionsCallbackTab[i].id == pListView->getID()) {
            return SListViewFunctionsCallbackTab[i].getListItemCountCallback(pListView);
            break;
        }
    }
    return 0;
}

void DeviceUpdateActivity::obtainListItemData(ZKListView *pListView,ZKListView::ZKListItem *pListItem, int index){
    int tablen = sizeof(SListViewFunctionsCallbackTab) / sizeof(S_ListViewFunctionsCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SListViewFunctionsCallbackTab[i].id == pListView->getID()) {
            SListViewFunctionsCallbackTab[i].obtainListItemDataCallback(pListView, pListItem, index);
            break;
        }
    }
}

void DeviceUpdateActivity::onItemClick(ZKListView *pListView, int index, int id){
    int tablen = sizeof(SListViewFunctionsCallbackTab) / sizeof(S_ListViewFunctionsCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SListViewFunctionsCallbackTab[i].id == pListView->getID()) {
            SListViewFunctionsCallbackTab[i].onItemClickCallback(pListView, index, id);
            break;
        }
    }
}

void DeviceUpdateActivity::onSlideItemClick(ZKSlideWindow *pSlideWindow, int index) {
    int tablen = sizeof(SSlideWindowItemClickCallbackTab) / sizeof(S_SlideWindowItemClickCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SSlideWindowItemClickCallbackTab[i].id == pSlideWindow->getID()) {
            SSlideWindowItemClickCallbackTab[i].onSlideItemClickCallback(pSlideWindow, index);
            break;
        }
    }
}

bool DeviceUpdateActivity::onTouchEvent(const MotionEvent &ev) {
    return onDeviceUpdateActivityTouchEvent(ev);
}

void DeviceUpdateActivity::onTextChanged(ZKTextView *pTextView, const std::string &text) {
    int tablen = sizeof(SEditTextInputCallbackTab) / sizeof(S_EditTextInputCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SEditTextInputCallbackTab[i].id == pTextView->getID()) {
            SEditTextInputCallbackTab[i].onEditTextChangedCallback(text);
            break;
        }
    }
}

void DeviceUpdateActivity::rigesterActivityTimer() {
    int tablen = sizeof(REGISTER_ACTIVITY_TIMER_TAB) / sizeof(S_ACTIVITY_TIMEER);
    for (int i = 0; i < tablen; ++i) {
        S_ACTIVITY_TIMEER temp = REGISTER_ACTIVITY_TIMER_TAB[i];
        registerTimer(temp.id, temp.time);
    }
}


void DeviceUpdateActivity::onVideoPlayerMessage(ZKVideoView *pVideoView, int msg) {
    int tablen = sizeof(SVideoViewCallbackTab) / sizeof(S_VideoViewCallback);
    for (int i = 0; i < tablen; ++i) {
        if (SVideoViewCallbackTab[i].id == pVideoView->getID()) {
        	if (SVideoViewCallbackTab[i].loop) {
                //循环播放
        		videoLoopPlayback(pVideoView, msg, i);
        	} else if (SVideoViewCallbackTab[i].onVideoViewCallback != NULL){
        	    SVideoViewCallbackTab[i].onVideoViewCallback(pVideoView, msg);
        	}
            break;
        }
    }
}

void DeviceUpdateActivity::videoLoopPlayback(ZKVideoView *pVideoView, int msg, size_t callbackTabIndex) {

	switch (msg) {
	case ZKVideoView::E_MSGTYPE_VIDEO_PLAY_STARTED:
		LOGD("ZKVideoView::E_MSGTYPE_VIDEO_PLAY_STARTED\n");
    if (callbackTabIndex >= (sizeof(SVideoViewCallbackTab)/sizeof(S_VideoViewCallback))) {
      break;
    }
		pVideoView->setVolume(SVideoViewCallbackTab[callbackTabIndex].defaultvolume / 10.0);
		mVideoLoopErrorCount = 0;
		break;
	case ZKVideoView::E_MSGTYPE_VIDEO_PLAY_ERROR:
		/**错误处理 */
		++mVideoLoopErrorCount;
		if (mVideoLoopErrorCount > 100) {
			LOGD("video loop error counts > 100, quit loop playback !");
            break;
		} //不用break, 继续尝试播放下一个
	case ZKVideoView::E_MSGTYPE_VIDEO_PLAY_COMPLETED:
		LOGD("ZKVideoView::E_MSGTYPE_VIDEO_PLAY_COMPLETED\n");
        std::vector<std::string> videolist;
        std::string fileName(getAppName());
        if (fileName.size() < 4) {
             LOGD("getAppName size < 4, ignore!");
             break;
        }
        fileName = fileName.substr(0, fileName.length() - 4) + "_video_list.txt";
        fileName = "/mnt/extsd/" + fileName;
        if (!parseVideoFileList(fileName.c_str(), videolist)) {
            LOGD("parseVideoFileList failed !");
		    break;
        }
		if (pVideoView && !videolist.empty()) {
			mVideoLoopIndex = (mVideoLoopIndex + 1) % videolist.size();
			pVideoView->play(videolist[mVideoLoopIndex].c_str());
		}
		break;
	}
}

void DeviceUpdateActivity::startVideoLoopPlayback() {
    int tablen = sizeof(SVideoViewCallbackTab) / sizeof(S_VideoViewCallback);
    for (int i = 0; i < tablen; ++i) {
    	if (SVideoViewCallbackTab[i].loop) {
    		ZKVideoView* videoView = (ZKVideoView*)findControlByID(SVideoViewCallbackTab[i].id);
    		if (!videoView) {
    			return;
    		}
    		//循环播放
    		videoLoopPlayback(videoView, ZKVideoView::E_MSGTYPE_VIDEO_PLAY_COMPLETED, i);
    		return;
    	}
    }
}

void DeviceUpdateActivity::stopVideoLoopPlayback() {
    int tablen = sizeof(SVideoViewCallbackTab) / sizeof(S_VideoViewCallback);
    for (int i = 0; i < tablen; ++i) {
    	if (SVideoViewCallbackTab[i].loop) {
    		ZKVideoView* videoView = (ZKVideoView*)findControlByID(SVideoViewCallbackTab[i].id);
    		if (!videoView) {
    			return;
    		}
    		if (videoView->isPlaying()) {
    		    videoView->stop();
    		}
    		return;
    	}
    }
}

bool DeviceUpdateActivity::parseVideoFileList(const char *pFileListPath, std::vector<string>& mediaFileList) {
	mediaFileList.clear();
	if (NULL == pFileListPath || 0 == strlen(pFileListPath)) {
        LOGD("video file list is null!");
		return false;
	}

	ifstream is(pFileListPath, ios_base::in);
	if (!is.is_open()) {
		LOGD("cann't open file %s \n", pFileListPath);
		return false;
	}
	char tmp[1024] = {0};
	while (is.getline(tmp, sizeof(tmp))) {
		string str = tmp;
		removeCharFromString(str, '\"');
		removeCharFromString(str, '\r');
		removeCharFromString(str, '\n');
		if (str.size() > 1) {
     		mediaFileList.push_back(str.c_str());
		}
	}
  LOGD("(f:%s, l:%d) parse fileList[%s], get [%d]files", __FUNCTION__,
      __LINE__, pFileListPath, int(mediaFileList.size()));
  for (std::vector<string>::size_type i = 0; i < mediaFileList.size(); i++) {
    LOGD("file[%u]:[%s]", ::size_t(i), mediaFileList[i].c_str());
  }
	is.close();

	return true;
}

int DeviceUpdateActivity::removeCharFromString(string& nString, char c) {
    string::size_type   pos;
    while(1) {
        pos = nString.find(c);
        if(pos != string::npos) {
            nString.erase(pos, 1);
        } else {
            break;
        }
    }
    return (int)nString.size();
}

void DeviceUpdateActivity::registerUserTimer(int id, int time) {
	registerTimer(id, time);
}

void DeviceUpdateActivity::unregisterUserTimer(int id) {
	unregisterTimer(id);
}

void DeviceUpdateActivity::resetUserTimer(int id, int time) {
	resetTimer(id, time);
}